from flask import Flask, jsonify, current_app, request
import config
import requests
import logging
import time
import socket
import slixmpp
import sys
import asyncio

logging.basicConfig(filename=config.owncast_logfile, level=logging.INFO)

app = Flask(__name__)


def get_now():  # Create a timestamp for logging
    now = str(time.strftime("%Y/%m/%d %H:%M:%S"))
    return now

class SendMsgBot(slixmpp.ClientXMPP):
    def __init__(self, jid, password, recipient, message):
        slixmpp.ClientXMPP.__init__(self, config.jid, config.password)
        self.recipient = recipient
        self.msg = message
        self.add_event_handler("session_start", self.start)

    async def start(self, event):
        self.send_presence()
        await self.get_roster()
        self.send_message(mto=self.recipient, mbody=self.msg, mtype='chat')
        self.disconnect()


def xmpp_chat(ownchat_msg):  # Send messages to XMPP
    xmpp = SendMsgBot(config.jid, config.password, config.to, ownchat_msg)
    xmpp.register_plugin('xep_0030')  # Service Discovery
    xmpp.register_plugin('xep_0199')  # XMPP Ping
    xmpp.connect()
    asyncio.get_event_loop().run_until_complete(xmpp.disconnected)


@app.route('/user_left/', methods=["POST"])  # Notify XMPP that a user left
def left():
    logging.info(f'{get_now()} - user_left request')
    raw_data = request.get_json(force=True)  # Get the raw data
    event_data = raw_data['eventData']
    logging.info(f'{get_now()} - {raw_data}')
    response = requests.get(f'{config.api_url}/api/status')
    response = response.json()
    if response['online'] is True:  # Check if we are currently streaming, if so, send notification
        chatter_name = event_data['user']['displayName']
        ownchat_msg = f'{chatter_name} left the chat.'
        logging.info(f'{get_now()} - {ownchat_msg}')
        xmpp_chat(ownchat_msg)
    else:  # If not currently streaming, do nothing.
        logging.info(f'{get_now()} - We are not streaming, so not sending XMPP message.')
    return jsonify({"Data": raw_data, })


@app.route('/user_joined/', methods=["POST"])  # Notify XMPP that a user joined
def joined():
    logging.info(f'{get_now()} - user_joined request')
    raw_data = request.get_json(force=True)  # Get the raw data
    event_data = raw_data['eventData']
    chatter_name = event_data['user']['displayName']
    ownchat_msg = f'{chatter_name} joined the chat.'
    logging.info(f'{get_now()} - {ownchat_msg}')
    logging.info(f'{get_now()} - {raw_data}')
    xmpp_chat(ownchat_msg)
    return jsonify({"Data": raw_data, })


@app.route('/name_changed/', methods=["POST"])  # Notify XMPP that a user changed their name
def changed():
    logging.info(f'{get_now()} - name_changed request')
    raw_data = request.get_json(force=True)  # Get the raw data
    event_data = raw_data['eventData']
    chatter_old_name = event_data['user']['previousNames']
    chatter_new_name = event_data['user']['displayName']
    last_name = len(chatter_old_name) - 1  # Get last name in previousNames list
    chatter_old_name = event_data['user']['previousNames'][last_name]
    ownchat_msg = f'{chatter_old_name} changed their name to {chatter_new_name}'
    logging.debug(f'{get_now()} - {type}\n{raw_data}')
    logging.info(f'{get_now()} - {ownchat_msg}')
    xmpp_chat(ownchat_msg)
    return jsonify({"Data": raw_data, })


@app.route('/message_sent/', methods=["POST"])  # Notify XMPP that a user sent a message
def sent():
    logging.info(f'----------------------------------------------------------------------------')
    logging.info(f'{get_now()} - message_sent request')
    raw_data = request.get_json(force=True)  # Get the raw data
    event_data = raw_data['eventData']
    chatter_name = event_data['user']['displayName']
    chatter_id = event_data['user']['id']
    chat_msg = event_data['rawBody']
    ownchat_msg = f'{chatter_name}: {chat_msg}'
    logging.info(f'{get_now()} - Chat message: \"{ownchat_msg}\".')
    xmpp_chat(ownchat_msg)
    return jsonify({"Data": raw_data, })


if __name__ == '__main__':
    try:
        app.run()
    except Exception as main_error:
        logging.info(f'{get_now()} - {main_error}')
